#!/usr/bin/env python3

from argparse import ArgumentParser
from dataclasses import dataclass
from dns.resolver import resolve_address, NXDOMAIN, NoAnswer, LifetimeTimeout, NoNameservers
from dns.exception import SyntaxError
from uvicorn import run
from fastapi import FastAPI, HTTPException, Request, Response
from fastapi.middleware.cors import CORSMiddleware
from slowapi.errors import RateLimitExceeded
from slowapi import Limiter, _rate_limit_exceeded_handler
from slowapi.util import get_remote_address

limiter = Limiter(key_func=get_remote_address)
app = FastAPI()
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)
app.state.limiter = limiter
app.add_exception_handler(RateLimitExceeded, _rate_limit_exceeded_handler)

limit = "1/second"

@app.get("/api/rdns/{ip}")
@limiter.limit(lambda: limit)
async def get_rdns(ip: str, request: Request, response: Response):
    try:
        answer = resolve_address(ip, search = True)
    except SyntaxError:
        raise HTTPException(status_code=400, detail="Invalid IP address")
    except (NXDOMAIN, NoAnswer):
        raise HTTPException(status_code=404, detail="No rDNS information found for IP")
    except LifetimeTimeout:
        raise HTTPException(status_code=504, detail="Request for rDNS information timed out")
    except NoNameservers:
        raise HTTPException(status_code=503, detail="No nameservers currently available to fulfil request")
    except:
        raise HTTPException(status_code=500, detail="Unexpected error occurred")
    response.headers["Cache-Control"] = f"max-age={answer.rrset.ttl}"
    return { "ip": ip, "rdns": str(answer[0]).rstrip(".") }

@dataclass
class IpApiArgs:
    address: str
    port: int
    limit: str

def parse_list_arg(arg: str):
    return [x.strip().lower() for x in arg.split(",") if x.strip()]

def main():
    global limit
    parser = ArgumentParser("ipmap")
    parser.add_argument("-a", "--address", default = "127.0.0.1", help = "the address to use for the api (default: %(default)s)")
    parser.add_argument("-p", "--port", default = 8000, type = int, help = "the port to use for the api (default: %(default)s)")
    parser.add_argument("-l", "--limit", default = limit, help = "the rate limit for the api (default: %(default)s)")
    args = parser.parse_args(namespace = IpApiArgs)
    limit = args.limit
    run(app, host=args.address, port=args.port)

if __name__ == "__main__":
    main()
